#The purpose of this script is to calculate the fatigue damage for all cases, 
#headings, cement levels and hotspots, both for the whole operation and
#for each individual sea state. The script imports stress ranges found
#from RFC using the WAFO package in Matlab, and uses SN curves and
#Miner-Palmgren summation to find the fatigue damage.

#Written by John Marius Hegseth, M.Sc. thesis spring 2014.

import numpy as np

e = open('fatiguedamage.dat', 'w')

for case in ['base', 'diffracted', 'damped', 'dragcoeff']:
    e.write('Case: %s\n' % case)
    for heading in [0, 90]:
        e.write('Heading: %d\n' % heading)
        for cement in [0, 2, 5, 10]:
            e.write('Cement shortfall: %d\n' % cement)
            for hotspot in [1, 2, 3, 4, 5]:

                if hotspot == 1:
                    t = 33.2
                    log_a1 = 15.117
                    log_a2 = 17.146
                    m1 = 4.0
                    m2 = 5.0
                    stress_limit = 106.97
                    k = 0
                elif hotspot == 2:
                    t = 33.2
                    log_a1 = 12.449
                    log_a2 = 16.081
                    m1 = 3.0
                    m2 = 5.0
                    stress_limit = 65.5
                    k = 0.15
                elif hotspot == 3:
                    t = 25.4
                    log_a1 = 11.546
                    log_a2 = 14.576
                    m1 = 3.0
                    m2 = 5.0
                    stress_limit = 32.75
                    k = 0.25
                elif hotspot == 4:
                    t = 25.4
                    log_a1 = 15.117
                    log_a2 = 17.146
                    m1 = 4.0
                    m2 = 5.0
                    stress_limit = 106.97
                    k = 0
                elif hotspot == 5:
                    t = 33.2
                    log_a1 = 15.117
                    log_a2 = 17.146
                    m1 = 4.0
                    m2 = 5.0
                    stress_limit = 106.97
                    k = 0

                f = open('Stressranges/ranges_%s%d_HS%d_cement%d.dat' % (case, heading, hotspot, cement), 'r')

                Histo = []
                D = np.zeros((1,122)) #Damage
                seastate = 0

                for line in f:
                    if line[0] == '*':
                        n, bins = np.histogram(Histo, bins=100)

                        for i in xrange(len(n)):
                            S = (bins[i]+bins[i+1])/2.0
                            
                            if S <= stress_limit:
                                N = 10.0**(log_a2 - m2*np.log10(S*(t/25.0)**k))
                                D[0][seastate] += n[i]/N
                            else:
                                N = 10.0**(log_a1 - m1*np.log10(S*(t/25.0)**k))
                                D[0][seastate] += n[i]/N

                        Histo = []
                        seastate += 1
                        continue

                    Histo.append(float(line)/1000000)

                f.close()

                g = open('seastate_probabilities.dat', 'r')

                weighted_D = np.zeros((1,122)) #Weighted Damage
                seastate = 0

                for line in g:
                    li = line.split()
                    weighted_D[0][seastate] = float(li[0])*D[0][seastate]
                    seastate += 1

                g.close()

                total_seastates = 2647

                damage = total_seastates*3.0*np.sum(weighted_D)) #Multiply by 3 because simulation is one hour, and a sea state is three hours
                e.write('%.2f\n' % life)

                h = open('damage_per_seastate_%s%d_HS%d_cement%d.dat' % (case, heading, hotspot, cement), 'w')

                for i in xrange(len(D[0])):
                    D_seastate = 3.0 * D[0][i] #Multiply by 3 because simulation is one hour, and a sea state is three hours
                    h.write('%e\n' % D_seastate)

                h.close()

e.close()