#The purpose of this script is to use conductor displacements from Abaqus to estimate 
#material soil damping using a Ramberg-Osgood model and the method described in
#Gazetas, G. and Dobry, R. (1984), 'Horizontal response of piles in layered soils'.

#Written by John Marius Hegseth, M.Sc. thesis spring 2014.

import matplotlib
import numpy as np

D = 0.9144 #Pipe diameter [m]
v = 0.4 #Poisson's ratio soil [-]
gb = [] #Shear strain and damping ratio [-]
G_Gmax = [] #Shear modulus ratio [-]

alpha = 1.0
C1 = 0.5
R = 1.7
tau_max = 20.0
G_max = 10000.0
gamma_r = tau_max/G_max

for i in xrange(1,21):
	G_Gmax.append(0.05 * i)
	
	gb.append((((1.0 - G_Gmax[i-1]) / (alpha * (G_Gmax[i-1]**R) * (1/C1)**(R-1.0)))**(1.0/(R-1.0)) * gamma_r, 2.0 / np.pi * (R-1.0) / (R+1.0) * (1.0-G_Gmax[i-1])))

D = 0.9144 #Pipe diameter [m]
v = 0.4 #Poisson's ratio soil [-]
xstat = [] #Displacement after preload [m]

g = open('dampingcoefficients.dat', 'w')

for frame in xrange(1,22):
	z = [] #Depth relative to mudline [m]
	xtot = [] # Total displacement [m]
	x = [] #Lateral displacement from riser response [m]
	gamma = [] #Average shear strain [-]
	beta = [] #Damping ratio relative to critical [-]

	f = open('displacements.dat', 'r')
	count = 1

	for line in f:
		if line[0] == '*':
			count += 1
			continue
		
		if frame == 1:
			li = line.split()
			xstat.append(float(li[1]))
			continue

		if frame == count:
			li = line.split()
			z.append(float(li[0]) + 4.0)
			xtot.append(float(li[1]))
			gamma.append(abs((1+v)/(2.5*D) * float(li[1])))

	f.close()

	if frame == 1:
		continue
	else:
		for i in xrange(len(xtot)):
			x.append(xtot[i]-xstat[i])

		for i in xrange(len(gamma)):
				for j in xrange(len(gb)-1):
					g_high = float(gb[j][0])
					g_low = float(gb[j+1][0])
					b_high = float(gb[j][1])
					b_low = float(gb[j+1][1])
					if float(gamma[i])>=g_low and float(gamma[i])<g_high:
						beta.append((b_low+(b_high-b_low)*(float(gamma[i])-g_low)/(g_high-g_low)))

		def soil(z,D):
		    c = 20.0 #kPa
		    gamma = 21.3 #kN/m^3
		    J = 0.25
		    eps_c = 0.01
		    y_c = 2.5*eps_c*D
		    X_r = 6*D / (gamma*D/c + J)

		    if z < X_r:
		        p_u = 3*c + gamma*z + J*c*z/D
		        p = [0.0, 0.23*p_u, 0.33*p_u, 0.5*p_u, 0.72*p_u, 0.72*p_u*z/X_r, 0.72*p_u*z/X_r, 0.72*p_u*z/X_r, 0.72*p_u*z/X_r]
		        y = [0.0, 0.1*y_c, 0.3*y_c, 1.0*y_c, 3.0*y_c, 15.0*y_c, 50*y_c, 100*y_c, 1000*y_c]
		        return p, y

		    else:
		        p_u = 9*c
		        p = [0.0, 0.23*p_u, 0.33*p_u, 0.5*p_u, 0.72*p_u, 0.72*p_u, 0.72*p_u, 0.72*p_u, 0.72*p_u]
		        y = [0.0, 0.1*y_c, 0.3*y_c, 1.0*y_c, 3.0*y_c, 15.0*y_c, 50*y_c, 100*y_c, 1000*y_c]
		        return p, y

		T = 9.5
		omega = 2 * np.pi / T
		force = []
		c_m = []

		for i in xrange(len(z)):
			p, y = soil(abs(float(z[i])),D)

			for j in xrange(len(p)-1):
					y_low = float(y[j])
					y_high = float(y[j+1])
					p_low = float(p[j])*1000*0.5*D
					p_high = float(p[j+1])*1000*0.5*D
					if abs(float(x[i]))>=y_low and abs(float(x[i]))<y_high:
						force.append((p_low+(p_high-p_low)*(abs(float(x[i]))-y_low)/(y_high-y_low)))

			if abs(float(x[i])) != 0.0:
				c_m.append(2.0*(force[i]/abs(x[i]))*beta[i]/omega)
			else:
				c_m.append(0.0)

		C = 0

		for i in xrange(len(z)):
			if i == 0:
				C += c_m[i]*abs(x[i]/x[0])*(abs(z[i+1])-abs(z[i]))/2.0
			elif i == (len(z)-1):
				C += c_m[i]*abs(x[i]/x[0])*(abs(z[i])-abs(z[i-1]))/2.0
			else:
				C += c_m[i]*abs(x[i]/x[0])*(abs(z[i+1])-abs(z[i-1]))/2.0

		g.write('%.2f\n' % C)

g.close()