#The purpose of this script is to read the displacements along the conductor
#from Abaqus for each load increment, and write them to file.

#Made by John Marius Hegseth, M.Sc. thesis spring 2014.

from abaqus import *

filename = ''

myOdb = session.openOdb(
    name=filename, 
    readOnly=False)

xmin = -0.5
xmax = -0.37
ymin = -0.0001
ymax = 0.0001
zmin = -16.0001
zmax = 0.0

myInstance = myOdb.rootAssembly.instances['CONDUCTOR-1']

nodeSet1 = []
nodeSet2 = []

for i in xrange(len(myInstance.nodes)):
	if myInstance.nodes[i].coordinates[0] < -0.38 and myInstance.nodes[i].coordinates[1] <= -4.0:
		nodeSet1.append(myInstance.nodes[i].label)

nodes1 = myInstance.NodeSetFromNodeLabels(name='DispNodes1', nodeLabels=nodeSet1)

myInstance = myOdb.rootAssembly.instances['BEAMCONDUCTOR-1']

for i in xrange(1,len(myInstance.nodes)):
	nodeSet2.append(myInstance.nodes[i].label)

nodes2 = myInstance.NodeSetFromNodeLabels(name='DispNodes2', nodeLabels=nodeSet2)

f = open('displacements.dat', 'w')

for frame in myOdb.steps['ResponseLoad'].frames:
	disp1 = frame.fieldOutputs['U'].getSubset(region=nodes1)
	disp2 = frame.fieldOutputs['U'].getSubset(region=nodes2)
	i = 0
	j = 0
	for v in disp1.values:
		f.write('%.2f %.6f\n' % (nodes1.nodes[i].coordinates[1], v.data[0]))
		i += 1
	for w in disp2.values:
		f.write('%.2f %.6f\n' % (nodes2.nodes[j].coordinates[1], w.data[0]))
		j += 1

	f.write('*\n')

f.close()